<?php
/*
Plugin Name: Piwishadow
Version: 0.1.7
Description: Display pictures, videos and piwigo pages in modal overlay window.
Plugin URI: http://piwigo.org/ext/extension_view.php?eid=521
Author: Binaryworld
Author URI: http://binaryworld.hd.free.fr/
*/
if (!defined('PHPWG_ROOT_PATH')) die('Hacking attempt!');
define('PIWISHADOW_PATH' , PHPWG_PLUGINS_PATH . basename(dirname(__FILE__)) . '/');
if (!defined( 'DS')) define( 'DS', DIRECTORY_SEPARATOR );
load_language('plugin.lang', PIWISHADOW_PATH);

include_once PIWISHADOW_PATH .'modal_overlay_lib.php';

class Piwishadow {

	public static $PLUGIN_ID = NULL;
	private static $_instance = NULL;
	
	public $config;
	private $_pluginPath;
	private $_pluginURL;
	private $_overlayLibs = array();

    /**
     * Prevent direct object creation
     */
    final private function  __construct() { }

    /**
     * Prevent object cloning
     */
    final private function  __clone() { }

    final public static function getInstance($pluginId){
        if(null !== self::$_instance){
            return self::$_instance;
        }
        $oldInstance = get_plugin_data($pluginId);
        
        self::$_instance = new Piwishadow();
		self::$PLUGIN_ID = $pluginId;
		self::$_instance->loadConfig();
		set_plugin_data(self::$PLUGIN_ID, self::$_instance);
        return self::$_instance;
    }	

	
	public function getPluginPath() {
		if ( !isset($this->_pluginPath) ) $this->_pluginPath = dirname(__FILE__);
		return $this->_pluginPath;
	}
	public function getPluginURL() {
		if ( !isset($this->_pluginURL) ) $this->_pluginURL =  get_root_url() .'plugins/'. basename(dirname(__FILE__));
		return $this->_pluginURL;
	}
	public function getConfig() {
		return $this->config;
	}

	
	public function getDefaultConfig() {
		$config = (object) array (
			'modalOverlayLib' 	=> ModalOverlayLibFactory::getDefaultOverlayLib(),
			'modalOverlayTheme'	=> ModalOverlayLibFactory::getDefaultOverlayTheme(),
			'mediaFilePattern'	=> '/^(?<path>.*)\/(?<file>[^\/]*)_-_(?<media>flv|mp4|swf)(?:_(?<width>\d+)x(?<height>\d+))?/i',
			//
			// Configuration for the piwigo content page
			//
			'content' 			=> (object) array (
				// Display the HD picture in shadowbox when the picture is clicked in the picture content page
				'displayHdPictureInOverlay'		=> true,
				'hdPicturePrefilterPattern' 		=> '/href="javascript:phpWGOpenWindow.*"/i',
				'hdPicturePrefilterReplacement' 	=> 'href="{$high.U_HIGH}" rel="lightbox"',
				// Display media player in the piwigo default content page 
				'displayMediaPlayer'				=> true
			),
			//
			// Configuration for the piwigo slideshow
			//
			'slideshow' 		=> (object) array (
				'displayInOverlay'				=> true,
				'prefilterPattern' 					=> '/(href="\{\$(?:U_SLIDESHOW|U_SLIDESHOW_START)\}")([^>]*)(rel=".*")?/i',
				'prefilterReplacement' 				=> '$1 rel="lightbox[diaporama 80% 80%]"$2',
				'closePattern' 						=> '/href="\{\$U_SLIDESHOW_STOP\}"([^>]*)/i',
				'closeReplacement' 					=> 'href="#" onclick="parent.window.PiwishadowOverlayWnd.close(); return false;"$1'
		),
			//
			// Configuration for the piwigo thumbnail
			//
			'thumbnail' 		=> (object) array (
				'displayContentInOverlay'			=> true,
				'displayMediaPlayerIcon'			=> true,
				'displayContentIcon'				=> true,
				'displayThumbLabel'					=> false,
				'displayContentPageInOverlay'		=> false,
				'buttonsTemplateSearch'				=> '{if isset($thumbnail.NAME)}{$thumbnail.NAME}{/if}',
				'hrefContentSearch'					=> 'href="{$thumbnail.URL}"'
		));
		
		return $config;
	}
	
    public function loadConfig() {
	
        $x = @file_get_contents( $this->getPluginPath() .DS.'data.dat' );
        if ($x!==false) {
            $c = unserialize($x);
            if (!empty($c)) {
	            //TODO: do some more tests here
	            $this->config = $c;
            }
        }
 
        if ( empty($this->config) ) {
			//
			// Piwishadow configuration
			//
			
			$this->config = $this->getDefaultConfig();
			
            $this->saveConfig();
        }
    }
    public function saveConfig()
    {
        $file = fopen( $this->getPluginPath() .'/data.dat', 'w' );
        fwrite($file, serialize($this->config) );
        fclose( $file );
    }
    
    //
    // Modal overlay lib
    //
    public function getOverlayLib($libName = null) {
    	if (empty($libName)) 
    		$libName = $this->config->modalOverlayLib;
    	
    	if ( !empty( $this->_overlayLibs[$libName] )) {
			return	$this->_overlayLibs[$libName];		
    	}
    	$this->_overlayLibs[$libName] = ModalOverlayLibFactory::createModalOverlayLib($libName);
    	return $this->_overlayLibs[$libName];
    }

    public function getInstalledOverlayLibs() {
    	return ModalOverlayLibFactory::getInstalledOverlayLibs();
    }
    
    //
    // Resources
    //
    /*
    public function getIconTemplate($name, $ext = '.png') {
    	$file = {$ROOT_URL}{$themeconf.icon_dir}/
    	if play.png
    }
    */

    
	//
	// Admin
	//
	public function onAdminMenu($menu) {
        array_push($menu,
            array(
                'NAME' => 'Piwishadow',
                'URL' => get_admin_plugin_menu_link(dirname(__FILE__).'/admin/piwishadow_admin.php')
            )
        );
        return $menu;
		
	}

	//
	// Site Event Management
	//
	
	public function onBeginPicture() {
		global $template;
		if ($this->config->content->displayHdPictureInOverlay === true) {
			$template->set_prefilter('default_content', 'piwishadow_pictureprefilter');
		}
		if ($this->config->slideshow->displayInOverlay === true) {
			$template->set_prefilter('picture', 'piwishadow_slideshowprefilter');
			$template->set_prefilter('slideshow', 'piwishadow_closeslideshowprefilter');
		}
	}
	
	public function onBeginIndex() {
		global $template;
		$test = $this->config;
		
		if ($this->config->thumbnail->displayContentInOverlay === true) {
			$template->set_prefilter('index_thumbnails', 'piwishadow_thumbprefilter');
		}
		if ($this->config->slideshow->displayInOverlay === true) { 
			$template->set_prefilter('index', 'piwishadow_slideshowprefilter');
			$template->set_prefilter('slideshow', 'piwishadow_closeslideshowprefilter');
		}
	}
	
	protected function assignPluginTemplateVars($template) {
		$template->assign('piwishadow', array(
			'displayMediaPlayerIcon' => $this->config->thumbnail->displayMediaPlayerIcon,
			'displayContentIcon' => $this->config->thumbnail->displayContentIcon,
			'displayThumbLabel' => $this->config->thumbnail->displayThumbLabel,
			'displayContentPageInOverlay' => $this->config->thumbnail->displayContentPageInOverlay,
			'modalOverlayLib' => $this->config->modalOverlayLib,
			'pluginPath' => $this->getPluginPath(),
			'pluginUrl' => $this->getPluginUrl()
		));
		
	}
	
	protected function addVideoContentTemplateVars(&$tpl_var, $matches) {
		global $picture;
		// Add piwishadow plugin paramters
		$tpl_var['U_PLAYER'] = $this->getOverlayLib()->getPlayerUrl();
		// TODO: add lib configruation tab in the plugin administration console
		$tpl_var['FLASH_VARS'] = $this->getOverlayLib()->getFlashVarsTemplate();
		
		// Add media parameters
		if (!empty($matches['path']) && !empty($matches['file']) && !empty($matches['media'])) {
			if (isset($tpl_var['NAME'])) $tpl_var['NAME'] = str_replace('_', ' ', $matches['file']);
			$tpl_var['MEDIA'] = strtolower($matches['media']);
			$filePath = ltrim($matches['path'], './');
			$tpl_var['VIDEO_URL'] = get_absolute_root_url(false) . $filePath .'/'. $matches['file'] .'.'. $matches['media'];
			if (!empty($matches['width'])) $tpl_var['VIDEO_WIDTH'] = $matches['width'];
			if (!empty($matches['height'])) $tpl_var['VIDEO_HEIGHT'] = $matches['height'];
		}
	}
	
	public function onEndIndexThumbnails($tpl_thumbnails_var) {
		global $template;
		$pattern = $this->config->mediaFilePattern;
		if (!empty($pattern)) {
			$nbrElements = count($tpl_thumbnails_var);
			for ($i = 0; $i < $nbrElements; $i++) {
				if (preg_match($pattern, $tpl_thumbnails_var[$i]['FILE_PATH'],$matches)) {
					$this->addVideoContentTemplateVars($tpl_thumbnails_var[$i], $matches);
					$tpl_thumbnails_var[$i]['REL_OVERLAY'] = $this->getOverlayLib()->getDefaultLinkRelationship();
					$tpl_thumbnails_var[$i]['U_OVERLAY'] = $tpl_thumbnails_var[$i]['VIDEO_URL'];
				} else { 
					$tpl_thumbnails_var[$i]['REL_OVERLAY'] = $this->getOverlayLib()->getDefaultLinkRelationship();
					$tpl_thumbnails_var[$i]['U_OVERLAY'] = $tpl_thumbnails_var[$i]['FILE_PATH'];
					if ($tpl_thumbnails_var[$i]['FILE_HAS_HD'] === true) {
						$pi = pathinfo($tpl_thumbnails_var[$i]['FILE_PATH']);
						$tpl_thumbnails_var[$i]['U_HIGH'] = $pi['dirname'].'/pwg_high/'.$pi['basename'];
					}
				}
			}
				
			$this->assignPluginTemplateVars($template);
			return $tpl_thumbnails_var;
		}
	}
	
	public function onRenderElementContent($content, $element_info) {
		global $page;
		$pattern = $this->config->mediaFilePattern;
		// Do not display video player in the slideshow
		if (!empty($pattern) && !$page['slideshow']) {
			if (preg_match($pattern, $element_info['image_url'], $matches)) {
				global $page, $template;
				$template->set_template_dir('./plugins/'. basename(dirname(__FILE__)) .'/template');
				$template->set_filenames(
					array('default_content'=>'video_content.tpl')
				);
				$params = array(
			      'SRC_IMG' => $element_info['image_url'],
			      'ALT_IMG' => $element_info['file'],
			      'WIDTH_IMG' => @$element_info['scaled_width'],
			      'HEIGHT_IMG' => @$element_info['scaled_height'],
				);
				$this->addVideoContentTemplateVars($params, $matches);
				$params['REL_OVERLAY'] = $this->getOverlayLib()->getDefaultLinkRelationship();
				$template->assign( $params );
				return $template->parse( 'default_content', true);
			}	
		}
		return $content;
	}
	
	public function onAppendHeader() {
		global $template;
		$piwishadowCSS = '<link rel="stylesheet" href="'. $this->getPluginURL() .'/piwishadow.css" type="text/css" media="screen" />';
		$template->append('head_elements', $piwishadowCSS);
		$this->getOverlayLib()->appendHeaderContent($template, 'head_elements', $this->config->modalOverlayTheme);
	}
} // End class	

$obj = Piwishadow::getInstance($plugin['id']);


// Event admin
add_event_handler('get_admin_plugin_menu_links', array(&$obj, 'onAdminMenu') );

// Event site
add_event_handler('loc_end_page_header', array(&$obj, 'onAppendHeader'));
add_event_handler('loc_begin_picture', array(&$obj, 'onBeginPicture'));
add_event_handler('loc_begin_index', array(&$obj, 'onBeginIndex'));
add_event_handler('loc_end_index_thumbnails', array(&$obj, 'onEndIndexThumbnails'));
if ($obj->config->content->displayMediaPlayer === true) {
	// add event handler for rendering element content EVENT_HANDLER_PRIORITY_NEUTRAL - 1 in order to be triggerd before the default handler
	add_event_handler('render_element_content', array(&$obj, 'onRenderElementContent'), EVENT_HANDLER_PRIORITY_NEUTRAL - 1, 2);
}

function piwishadow_pictureprefilter($content, &$smarty) {
	$obj = get_plugin_data(Piwishadow::$PLUGIN_ID);
	return preg_replace($obj->config->content->hdPicturePrefilterPattern, $obj->config->content->hdPicturePrefilterReplacement, $content);
}
function piwishadow_slideshowprefilter($content, &$smarty) {
	$obj = get_plugin_data(Piwishadow::$PLUGIN_ID);
	return preg_replace($obj->config->slideshow->prefilterPattern, $obj->config->slideshow->prefilterReplacement, $content);
}
function piwishadow_closeslideshowprefilter($content, &$smarty) {
	$obj = get_plugin_data(Piwishadow::$PLUGIN_ID);
	return preg_replace($obj->config->slideshow->closePattern, $obj->config->slideshow->closeReplacement, $content);
}
// Prefilter for thumbnail
function piwishadow_thumbprefilter($content, &$smarty) {
	static $thumbButtonsTemplate, $thumbContentTemplate;
	$obj = get_plugin_data(Piwishadow::$PLUGIN_ID);

	if ($obj->config->thumbnail->displayContentInOverlay === true ) {
	
		if (empty($thumbContentTemplate)) {
			$thumbContentTemplate = file_get_contents( $obj->getPluginPath() .DS. 'template' .DS. 'thumbnail_piwishadow_content.tpl' );
		}
		// replace all occurences
		$content = str_replace($obj->config->thumbnail->hrefContentSearch, $thumbContentTemplate, $content);
	}
		
	if ($obj->config->thumbnail->displayMediaPlayerIcon === true || $obj->config->thumbnail->displayContentIcon === true) {

		if (empty($thumbButtonsTemplate)) {
			$thumbButtonsTemplate = file_get_contents( $obj->getPluginPath() .DS. 'template' .DS. 'thumbnail_piwishadow_buttons.tpl' );
		}
		
		$count = 1; // replace only the first occurence
		$content = str_replace($obj->config->thumbnail->buttonsTemplateSearch, $thumbButtonsTemplate, $content, $count);
	}
	return $content;
}

?>